@@ -10,8 +10,9 @@ register_mlr3 = function() {
     lg$set_threshold("warn") # nolint
   }
   x = utils::getFromNamespace("mlr_reflections", ns = "mlr3")
-  x$task_col_roles$classif = c(x$task_col_roles$classif, "subset")
-  x$task_col_roles$regr = c(x$task_col_roles$regr, "subset")
+  for(task_type in names(x$task_col_roles)){
+    x$task_col_roles[[task_type]] = unique(c(x$task_col_roles[[task_type]], "subset"))
+  }
   x$loaded_packages = c(x$loaded_packages, "mlr3resampling")
   mlr3misc::register_namespace_callback(pkgname, "mlr3", register_mlr3)
 }
