@@ -1,5 +1,7 @@
 library(testthat)
 library(data.table)
+if(requireNamespace("lgr"))lgr::get_logger("mlr3")$set_threshold("warn")
+
 test_that("resampling error if no group", {
   itask <- mlr3::TaskClassif$new("iris", iris, target="Species")
   same_other <- mlr3resampling::ResamplingSameOtherCV$new()
@@ -549,69 +551,56 @@ test_that("regular K fold CV works in proj", {
       list(),
       score_args=mlr3::msrs(c("regr.rmse", "regr.mae")))
   }, "resamplings is empty, but need at least one")
-  mlr3resampling::proj_grid(
+  grid_dt <- mlr3resampling::proj_grid(
     pkg.proj.dir,
     reg.task.list,
     reg.learner.list,
     kfold,
+    save_learner = TRUE,
     score_args=mlr3::msrs(c("regr.rmse", "regr.mae")))
-  grid_jobs <- fread(file.path(pkg.proj.dir, "grid_jobs.csv"))
-  expect_true(all(grid_jobs$status=="not started"))
-  expected_jobs <- length(reg.learner.list)*length(reg.task.list)*kfold$param_set$values$folds
-  expect_equal(nrow(grid_jobs), expected_jobs)
-  results.csv <- file.path(pkg.proj.dir, "results.csv")
-  expect_false(file.exists(results.csv))
-  row1 <- mlr3resampling::proj_compute(pkg.proj.dir)
-  expect_equal(nrow(row1), 1)
-  mlr3resampling::proj_grid(
-    pkg.proj.dir,
-    reg.task.list,
-    reg.learner.list,
-    kfold,
-    score_args=mlr3::msrs(c("regr.rmse", "regr.mae")))
-  grid_jobs <- fread(file.path(pkg.proj.dir, "grid_jobs.csv"))
-  expect_identical(grid_jobs$status, c("done", rep("not started", expected_jobs-1)))
-  row2 <- mlr3resampling::proj_compute(pkg.proj.dir)
+  todo.i <- mlr3resampling::proj_todo(pkg.proj.dir)
+  todo.expected <- 1:nrow(grid_dt)
+  expect_identical(todo.i, todo.expected)
+  row2 <- mlr3resampling::proj_compute(2, pkg.proj.dir)
   expect_equal(nrow(row2), 1)
+  todo.i <- mlr3resampling:::proj_todo(pkg.proj.dir)
+  expect_identical(todo.i, todo.expected[-2])
+  row3 <- mlr3resampling::proj_compute(3, pkg.proj.dir)
   two_rows <- mlr3resampling::proj_results(pkg.proj.dir)
   expect_equal(nrow(two_rows), 2)
-  mlr3resampling::proj_compute_until_done(pkg.proj.dir)
+  mlr3resampling::proj_compute_all(pkg.proj.dir)
   expect_false(file.exists(file.path(pkg.proj.dir, "learners.csv")))
-  results_dt <- fread(results.csv)
-  expect_equal(nrow(results_dt), expected_jobs)
+  results_dt <- fread(file.path(pkg.proj.dir, "results.csv"))
+  expect_false(identical(results_dt$regr.mae[1], results_dt$regr.mae[2]))
+  expect_equal(nrow(results_dt), 8)
+  expect_error({
+    mlr3resampling::proj_grid(
+      pkg.proj.dir,
+      reg.task.list,
+      reg.learner.list,
+      kfold)
+  }, "already exists, so not over-writing")
+  pkg.proj.dir <- tempfile()
   expect_warning({
     mlr3resampling::proj_grid(
       pkg.proj.dir,
       reg.task.list,
       reg.learner.list,
       kfold)
-  }, "no score_args, nor save_learner, nor save_pred, so there will no results other than computation times")
-  kfold$param_set$values$folds <- 5
-  expect_warning({
-    mlr3resampling::proj_grid(
-      pkg.proj.dir,
-      reg.task.list,
-      reg.learner.list,
-      kfold,
-      score_args=mlr3::msrs(c("regr.rmse", "regr.mae")))
-  }, "grid_jobs.csv changed!")
+  }, "no score_args nor save_pred, so there will no test error results")
 })
 
-mlr3torch_available <- requireNamespace("mlr3torch") && torch::torch_is_installed()
-if(mlr3torch_available)test_that("mlr3torch history saved", {
-  N <- 80
+test_that("proj_test down-samples proportionally", {
+  N <- 8000
   set.seed(1)
-  people <- c("Alice","Bob")
   reg.dt <- data.table(
     x=runif(N, -2, 2),
-    person=factor(rep(people, each=0.5*N)))
+    person=factor(rep(c("Alice","Bob"), c(0.1,0.9)*N)))
   reg.pattern.list <- list(
     easy=function(x, person)x^2,
     impossible=function(x, person)(x^2)*(-1)^as.integer(person))
-  kfold <- mlr3resampling::ResamplingSameOtherSizesCV$new()
+  kfold <- mlr3::ResamplingCV$new()
   kfold$param_set$values$folds <- 2
-  subsets <- "SA"
-  kfold$param_set$values$subsets <- subsets
   reg.task.list <- list()
   for(pattern in names(reg.pattern.list)){
     f <- reg.pattern.list[[pattern]]
@@ -625,170 +614,139 @@ if(mlr3torch_available)test_that("mlr3torch history saved", {
     task.obj$col_roles$subset <- "person"
     reg.task.list[[pattern]] <- task.obj
   }
-  Tlrn <- mlr3torch::LearnerTorchMLP$new(task_type="regr")
-  mlr3::set_validate(Tlrn, validate = 0.5)
-  n.epochs <- 3
-  Tlrn$callbacks <- mlr3torch::t_clbk("history")
-  Tlrn$param_set$values$patience <- n.epochs
-  Tlrn$param_set$values$batch_size <- 10
-  Tlrn$param_set$values$epochs <- paradox::to_tune(upper = n.epochs, internal = TRUE)
-  Tlrn$param_set$values[c("measures_train","measures_valid")] <- mlr3::msrs(c("regr.rmse"))
   reg.learner.list <- list(
-    mlr3tuning::auto_tuner(
-      learner = Tlrn,
-      tuner = mlr3tuning::tnr("internal"),
-      resampling = mlr3::rsmp("insample"),
-      measure = mlr3::msr("internal_valid_score", minimize = TRUE),
-      term_evals = 1,
-      id="torch_linear",
-      store_models = TRUE),
+    featureless=mlr3::LearnerRegrFeatureless$new())
+  if(requireNamespace("rpart")){
+    reg.learner.list$rpart <- mlr3::LearnerRegrRpart$new()
+  }
+  pkg.proj.dir <- tempfile()
+  keep_cols <- c("var","dev","n")
+  pgrid <- mlr3resampling::proj_grid(
+    pkg.proj.dir,
+    reg.task.list,
+    reg.learner.list,
+    kfold,
+    save_learner=function(L){
+      if(inherits(L, "LearnerRegrRpart")){
+        list(rpart=L$model$frame[,keep_cols])
+      }
+    },
+    save_pred = TRUE,
+    score_args=mlr3::msrs(c("regr.rmse", "regr.mae")))
+  out_list <- mlr3resampling::proj_test(pkg.proj.dir)
+  expect_identical(names(out_list), c("grid_jobs.csv", "learners_rpart.csv", "results.csv"))
+  expect_identical(out_list$grid_jobs.csv$iteration, rep(1L, 4))
+  test_dir <- file.path(pkg.proj.dir, "test")
+  test.task <- readRDS(file.path(test_dir, "tasks", "1.rds"))
+  count.tab <- table(test.task$data(cols="person"))
+  expect_equal(as.numeric(count.tab), c(10, 90))
+  test_res_dt <- readRDS(file.path(test_dir, "results.rds"))
+  expect_equal(length(test_res_dt$pred[[1]]$row_ids), 50)
+  learners_dt <- fread(file.path(test_dir, "learners_rpart.csv"))
+  expected_csv_cols <- c("grid_job_i", "var", "dev", "n")
+  expect_identical(names(learners_dt), expected_csv_cols)
+  out_dt_list <- mlr3resampling::proj_fread(test_dir)
+  expected_join_cols <- c(
+    expected_csv_cols,
+    "task_id", "learner_id", "resampling_id", "iteration")
+  expect_identical(names(out_dt_list$learners_rpart.csv), expected_join_cols)
+  rpart.job.i.vec <- which(pgrid$learner_id=="regr.rpart")
+  mlr3resampling::proj_compute(rpart.job.i.vec[1], pkg.proj.dir)
+  mlr3resampling::proj_compute(rpart.job.i.vec[length(rpart.job.i.vec)], pkg.proj.dir)
+  mlr3resampling::proj_results_save(pkg.proj.dir)
+  csv_dt_list <- mlr3resampling::proj_fread(pkg.proj.dir)
+  rpart_dt <- csv_dt_list[["learners_rpart.csv"]]
+  expect_identical(names(rpart_dt), expected_join_cols)
+  expect_equal(sum(is.na(rpart_dt[["task_id"]])), 0)
+  expect_equal(sum(is.na(rpart_dt[["learner_id"]])), 0)
+  expect_equal(sum(is.na(rpart_dt[["resampling_id"]])), 0)
+  expect_equal(sum(is.na(rpart_dt[["iteration"]])), 0)
+})
+
+last_lev <- function(x){
+  levs <- levels(factor(x))
+  levs[length(levs)]
+}
+
+test_that("set works after score(), other is last Y level", {
+  N <- 80
+  set.seed(1)
+  reg.dt <- data.table(
+    x=runif(N, -2, 2),
+    person=rep(1:2, each=0.5*N))
+  reg.pattern.list <- list(
+    easy=function(x, person)x^2,
+    impossible=function(x, person)(x^2)*(-1)^person)
+  SOAK <- mlr3resampling::ResamplingSameOtherSizesCV$new()
+  reg.task.list <- list()
+  for(pattern in names(reg.pattern.list)){
+    f <- reg.pattern.list[[pattern]]
+    yname <- paste0("y_",pattern)
+    reg.dt[, (yname) := f(x,person)+rnorm(N, sd=0.5)][]
+    task.dt <- reg.dt[, c("x","person",yname), with=FALSE]
+    task.obj <- mlr3::TaskRegr$new(
+      pattern, task.dt, target=yname)
+    task.obj$col_roles$stratum <- "person"
+    task.obj$col_roles$subset <- "person"
+    reg.task.list[[pattern]] <- task.obj
+  }
+  reg.learner.list <- list(
     mlr3::LearnerRegrFeatureless$new())
-  pkg.proj.dir <- tempfile()
-  get_history <- function(L){
-    if(grepl("torch", L$id)){
-      V <- L$tuning_result$internal_tuned_values[[1]]
-      M <- L$archive$learners(1)[[1]]$model
-      M$callbacks$history
-    }
+  if(requireNamespace("rpart")){
+    reg.learner.list$rpart <- mlr3::LearnerRegrRpart$new()
   }
-  mlr3resampling::proj_grid(
-    pkg.proj.dir,
+  (bench.grid <- mlr3::benchmark_grid(
     reg.task.list,
     reg.learner.list,
-    kfold,
-    score_args=mlr3::msrs(c("regr.rmse", "regr.mae")),
-    save_learner = get_history)
-  from_rds <- readRDS(file.path(pkg.proj.dir, "grid_jobs.rds"))
-  expect_equal(from_rds[test.fold==1 & test.subset=="Bob" & train.subsets=="all" & task_id=="easy", unique(learner_id)], c("torch_linear", "regr.featureless"))
-  expect_equal(from_rds[learner.i==1, unique(n.train.groups)], c(40, 20))
-  grid_jobs <- fread(file.path(pkg.proj.dir, "grid_jobs.csv"))
-  expect_true(all(grid_jobs$status=="not started"))
-  expected_base <- length(reg.task.list)*kfold$param_set$values$folds*length(people)*nchar(subsets)
-  expected_epochs <- n.epochs*expected_base
-  expected_jobs <- length(reg.learner.list)*expected_base
-  expect_equal(nrow(grid_jobs), expected_jobs)
-  results.csv <- file.path(pkg.proj.dir, "results.csv")
-  expect_false(file.exists(results.csv))
-  row1 <- mlr3resampling::proj_compute(pkg.proj.dir)
-  expected_model_cols <- c("epoch","train.regr.rmse","valid.regr.rmse")
-  expect_train_valid <- function(R){
-    train_valid_rows <- R$learner[[1]]
-    expect_equal(nrow(train_valid_rows), n.epochs)
-    expect_identical(names(train_valid_rows), expected_model_cols)
-  }
-  expect_train_valid(row1)
-  from.disk <- mlr3resampling::proj_results(pkg.proj.dir)
-  expect_train_valid(from.disk)
-  mlr3resampling::proj_compute_until_done(pkg.proj.dir)
-  model_dt <- fread(file.path(pkg.proj.dir, "learners.csv"))
-  expected_cols <- c(
-    "task.i", "learner.i", "resampling.i", "iteration", "start.time", "end.time",
-    "process", "regr.rmse", "regr.mae",
-    "task_id", "learner_id", "resampling_id", "test.subset",
-    "train.subsets", "groups", "test.fold", "seed", "n.train.groups",
-    expected_model_cols)
-  expect_identical(names(model_dt), expected_cols)
-  expect_equal(nrow(model_dt), expected_epochs)
-  results_dt <- fread(file.path(pkg.proj.dir, "results.csv"))
-  pval_list <- mlr3resampling::pvalue(results_dt)
-  expect_is(pval_list, "pvalue")
-  expect_true(all(pval_list$pvalues$Train_subsets=="all-same"))
-  if(interactive())plot(pval_list)
+    SOAK))
+  bench.result <- mlr3::benchmark(bench.grid)
+  bench.score <- mlr3resampling::score(bench.result, mlr3::msr("regr.rmse"))
+  if(interactive())plot(bench.score)
+  set(bench.score, j="foo", value=1)
+  expect_is(bench.score, "score")
+  expect_identical(last_lev(bench.score$Train_subsets), "other")
+  bench.pvalue <- mlr3resampling::pvalue(bench.score)
+  if(interactive())plot(bench.pvalue)
+  expect_identical(last_lev(bench.pvalue$stats$Train_subsets), "other")
 })
 
-if(mlr3torch_available)test_that("mlr3torch history and weights saved", {
+test_that("plot ok without other", {
   N <- 80
   set.seed(1)
-  people <- c("Alice","Bob","Bob","Bob")
   reg.dt <- data.table(
     x=runif(N, -2, 2),
-    person=factor(rep(people, each=0.25*N)))
+    person=rep(1:2, each=0.5*N))
   reg.pattern.list <- list(
     easy=function(x, person)x^2,
-    impossible=function(x, person)(x^2)*(-1)^as.integer(person))
-  kfold <- mlr3resampling::ResamplingSameOtherSizesCV$new()
-  kfold$param_set$values$folds <- 2
-  kfold$param_set$values$sizes <- 1
-  subsets <- "S"
-  kfold$param_set$values$subsets <- subsets
+    impossible=function(x, person)(x^2)*(-1)^person)
+  SAK <- mlr3resampling::ResamplingSameOtherSizesCV$new()
+  SAK$param_set$values$subsets <- "SA"
   reg.task.list <- list()
   for(pattern in names(reg.pattern.list)){
     f <- reg.pattern.list[[pattern]]
-    task.dt <- data.table(reg.dt)[
-    , y := f(x,person)+rnorm(N, sd=0.5)
-    ][]
+    yname <- paste0("y_",pattern)
+    reg.dt[, (yname) := f(x,person)+rnorm(N, sd=0.5)][]
+    task.dt <- reg.dt[, c("x","person",yname), with=FALSE]
     task.obj <- mlr3::TaskRegr$new(
-      pattern, task.dt, target="y")
-    task.obj$col_roles$feature <- "x"
+      pattern, task.dt, target=yname)
     task.obj$col_roles$stratum <- "person"
     task.obj$col_roles$subset <- "person"
     reg.task.list[[pattern]] <- task.obj
   }
-  Tlrn <- mlr3torch::LearnerTorchMLP$new(task_type="regr")
-  mlr3::set_validate(Tlrn, validate = 0.5)
-  n.epochs <- 3
-  Tlrn$callbacks <- mlr3torch::t_clbk("history")
-  Tlrn$param_set$values$patience <- n.epochs
-  Tlrn$param_set$values$batch_size <- 10
-  Tlrn$param_set$values$epochs <- paradox::to_tune(upper = n.epochs, internal = TRUE)
-  Tlrn$param_set$values[c("measures_train","measures_valid")] <- mlr3::msrs(c("regr.rmse"))
   reg.learner.list <- list(
-    mlr3::LearnerRegrFeatureless$new(),
-    mlr3tuning::auto_tuner(
-      learner = Tlrn,
-      tuner = mlr3tuning::tnr("internal"),
-      resampling = mlr3::rsmp("insample"),
-      measure = mlr3::msr("internal_valid_score", minimize = TRUE),
-      term_evals = 1,
-      id="torch_linear",
-      store_models = TRUE))
-  pkg.proj.dir <- tempfile()
-  get_history_weights <- function(L){
-    if(grepl("torch", L$id)){
-      V <- L$tuning_result$internal_tuned_values[[1]]
-      M <- L$archive$learners(1)[[1]]$model
-      list(
-        history=M$callbacks$history,
-        weights=do.call(data.table, lapply(M$network$parameters, torch::as_array)))
-    }
+    mlr3::LearnerRegrFeatureless$new())
+  if(requireNamespace("rpart")){
+    reg.learner.list$rpart <- mlr3::LearnerRegrRpart$new()
   }
-  mlr3resampling::proj_grid(
-    pkg.proj.dir,
+  (bench.grid <- mlr3::benchmark_grid(
     reg.task.list,
     reg.learner.list,
-    kfold,
-    score_args=mlr3::msrs(c("regr.rmse", "regr.mae")),
-    save_learner = get_history_weights)
-  from_rds <- readRDS(file.path(pkg.proj.dir, "grid_jobs.rds"))
-  one_test_N <- from_rds[test.fold==1 & learner.i==1 & task.i==1, .(test.subset, n.train.groups)]
-  expect_equal(one_test_N$n.train.groups, c(30, 15, 10, 5))
-  grid_jobs <- fread(file.path(pkg.proj.dir, "grid_jobs.csv"))
-  expect_true(all(grid_jobs$status=="not started"))
-  expected_base <- length(reg.task.list)*kfold$param_set$values$folds*length(people)*nchar(subsets)
-  expected_epochs <- n.epochs*expected_base
-  expected_jobs <- length(reg.learner.list)*expected_base
-  expect_equal(nrow(grid_jobs), expected_jobs)
-  results.csv <- file.path(pkg.proj.dir, "results.csv")
-  expect_false(file.exists(results.csv))
-  row1 <- mlr3resampling::proj_compute(pkg.proj.dir)
-  expect_null(row1$learner[[1]])
-  mlr3resampling::proj_compute_until_done(pkg.proj.dir)
-  history_dt <- fread(file.path(pkg.proj.dir, "learners_history.csv"))
-  expected_cols <- c(
-    "task.i", "learner.i", "resampling.i", "iteration", "start.time", "end.time",
-    "process", "regr.rmse", "regr.mae",
-    "task_id", "learner_id", "resampling_id", "test.subset",
-    "train.subsets", "groups", "test.fold", "seed", "n.train.groups",
-    "epoch","train.regr.rmse","valid.regr.rmse")
-  expect_identical(names(history_dt), expected_cols)
-  expect_equal(nrow(history_dt), expected_epochs)
-  weights_dt <- fread(file.path(pkg.proj.dir, "learners_weights.csv"))
-  expected_cols <- c(
-    "task.i", "learner.i", "resampling.i", "iteration", "start.time", "end.time",
-    "process", "regr.rmse", "regr.mae",
-    "task_id", "learner_id", "resampling_id", "test.subset",
-    "train.subsets", "groups", "test.fold", "seed", "n.train.groups",
-    "0.weight.V1", "0.bias")
-  expect_identical(names(weights_dt), expected_cols)
-  expect_equal(nrow(weights_dt), expected_base)
+    SAK))
+  bench.result <- mlr3::benchmark(bench.grid)
+  bench.score <- mlr3resampling::score(bench.result, mlr3::msr("regr.rmse"))
+  if(interactive())plot(bench.score)
+  expect_identical(last_lev(bench.score$Train_subsets), "same")
+  bench.pvalue <- mlr3resampling::pvalue(bench.score)
+  if(interactive())plot(bench.pvalue)
+  expect_identical(last_lev(bench.pvalue$stats$Train_subsets), "same")
 })
